@@ -3,7 +3,7 @@
 *                           O p e n G L   O b j e c t                           *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXGLObject.h,v 1.26 2005/01/16 16:06:06 fox Exp $                        *
+* $Id: FXGLObject.h,v 1.28.2.2 2006/07/28 00:56:30 fox Exp $                        *
 ********************************************************************************/
 #ifndef FXGLOBJECT_H
 #define FXGLOBJECT_H
@@ -35,9 +35,6 @@ class FXGLViewer;
 class FXGLObject;
 
 
-// List of objects
-typedef FXObjectListOf<FXGLObject> FXGLObjectList;
-
 /// Basic OpenGL object
 class FXAPI FXGLObject : public FXObject {
   FXDECLARE(FXGLObject)
@@ -82,13 +79,12 @@ public:
   };
 
 
-// Suppress warning about FXGLGroup::list not being exported
-#ifdef _MSC_VER
-#if _MSC_VER >= 1200
-#pragma warning( push )
-#endif
-#pragma warning( disable : 4251 )
-#endif
+/// Explicit template specialization
+FXTEMPLATE_EXTERN template class FXAPI FXObjectListOf<FXGLObject>;
+
+
+/// List of GL objects
+typedef FXObjectListOf<FXGLObject> FXGLObjectList;
 
 
 /// Group object
@@ -150,7 +146,7 @@ public:
   void remove(FXGLObject* obj){ list.remove(obj); }
 
   /// Remove child object at given position
-  void remove(FXint pos){ list.remove(pos); }
+  void erase(FXint pos){ list.erase(pos); }
 
   /// Remove all children
   void clear(){ list.clear(); }
@@ -164,14 +160,6 @@ public:
   };
 
 
-// Restore previous warning levels
-#ifdef _MSC_VER
-#if _MSC_VER >= 1200
-#pragma warning( pop )
-#endif
-#endif
-
-
 /// OpenGL Point Object
 class FXAPI FXGLPoint : public FXGLObject {
   FXDECLARE(FXGLPoint)
