@@ -3,7 +3,7 @@
 *                        F i l e    L i s t   O b j e c t                       *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 1998,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 1998,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,12 +19,14 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFileList.cpp,v 1.173.2.1 2005/02/08 07:17:57 fox Exp $                     *
+* $Id: FXFileList.cpp,v 1.211.2.1 2006/08/01 18:04:33 fox Exp $                     *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "fxkeys.h"
+#include "fxascii.h"
+#include "fxunicode.h"
 #include "FXHash.h"
 #include "FXThread.h"
 #include "FXStream.h"
@@ -33,6 +35,9 @@
 #include "FXSize.h"
 #include "FXPoint.h"
 #include "FXRectangle.h"
+#include "FXSystem.h"
+#include "FXPath.h"
+#include "FXStat.h"
 #include "FXFile.h"
 #include "FXURL.h"
 #include "FXSettings.h"
@@ -44,13 +49,20 @@
 #include "FXScrollBar.h"
 #include "FXIconSource.h"
 #include "FXIconDict.h"
+#include "FXShell.h"
+#include "FXPopup.h"
+#include "FXMenuPane.h"
+#include "FXMenuCaption.h"
+#include "FXMenuCommand.h"
+#include "FXMenuCascade.h"
+#include "FXMenuRadio.h"
+#include "FXMenuCheck.h"
+#include "FXMenuSeparator.h"
 #include "FXFileDict.h"
 #include "FXHeader.h"
 #include "FXIconList.h"
 #include "FXFileList.h"
-#ifdef WIN32
-#include <shellapi.h>
-#endif
+#include "FXDir.h"
 #include "icons.h"
 
 /*
@@ -68,6 +80,8 @@
   - Should blending also happen in FXIconDict? Or more general solution.
   - If you land in a large directory with images, things are a tad slow;
     need to speed this up some how.
+  - Since '\0' is no longer special in FXString, perhaps we can replace the function
+    of '\t' with '\0'.  This would be significantly more efficient.
 */
 
 
@@ -81,6 +95,9 @@
 #define TIMEFORMAT "%m/%d/%Y %H:%M:%S"
 #endif
 
+
+
+
 using namespace FX;
 
 
@@ -132,9 +149,9 @@ FXDEFMAP(FXFileList) FXFileListMap[]={
   FXMAPFUNC(SEL_COMMAND,FXFileList::ID_SORT_CASE,FXFileList::onCmdSortCase),
   FXMAPFUNC(SEL_COMMAND,FXFileList::ID_SET_PATTERN,FXFileList::onCmdSetPattern),
   FXMAPFUNC(SEL_COMMAND,FXFileList::ID_SET_DIRECTORY,FXFileList::onCmdSetDirectory),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETVALUE,FXFileList::onCmdSetValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_SETSTRINGVALUE,FXFileList::onCmdSetStringValue),
-  FXMAPFUNC(SEL_COMMAND,FXWindow::ID_GETSTRINGVALUE,FXFileList::onCmdGetStringValue),
+  FXMAPFUNC(SEL_COMMAND,FXFileList::ID_SETVALUE,FXFileList::onCmdSetValue),
+  FXMAPFUNC(SEL_COMMAND,FXFileList::ID_SETSTRINGVALUE,FXFileList::onCmdSetStringValue),
+  FXMAPFUNC(SEL_COMMAND,FXFileList::ID_GETSTRINGVALUE,FXFileList::onCmdGetStringValue),
   FXMAPFUNC(SEL_COMMAND,FXFileList::ID_SHOW_HIDDEN,FXFileList::onCmdShowHidden),
   FXMAPFUNC(SEL_COMMAND,FXFileList::ID_HIDE_HIDDEN,FXFileList::onCmdHideHidden),
   FXMAPFUNC(SEL_COMMAND,FXFileList::ID_TOGGLE_HIDDEN,FXFileList::onCmdToggleHidden),
@@ -150,7 +167,7 @@ FXIMPLEMENT(FXFileList,FXIconList,FXFileListMap,ARRAYNUMBER(FXFileListMap))
 
 // For serialization
 FXFileList::FXFileList(){
-  flags|=FLAG_ENABLED|FLAG_DROPTARGET;
+  dropEnable();
 #ifdef WIN32
   matchmode=FILEMATCH_FILE_NAME|FILEMATCH_NOESCAPE|FILEMATCH_CASEFOLD;
   sortfunc=ascendingCase;
@@ -170,17 +187,17 @@ FXFileList::FXFileList(){
 // File List
 FXFileList::FXFileList(FXComposite *p,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h):
   FXIconList(p,tgt,sel,opts,x,y,w,h),directory(PATHSEPSTRING),orgdirectory(PATHSEPSTRING),pattern("*"){
-  flags|=FLAG_ENABLED|FLAG_DROPTARGET;
+  dropEnable();
   associations=NULL;
-  appendHeader("Name",NULL,200);
-  appendHeader("Type",NULL,100);
-  appendHeader("Size",NULL,60);
-  appendHeader("Modified Date",NULL,150);
-  appendHeader("User",NULL,50);
-  appendHeader("Group",NULL,50);
-  appendHeader("Attributes",NULL,100);
+  appendHeader(tr("Name"),NULL,200);
+  appendHeader(tr("Type"),NULL,100);
+  appendHeader(tr("Size"),NULL,60);
+  appendHeader(tr("Modified Date"),NULL,150);
+  appendHeader(tr("User"),NULL,50);
+  appendHeader(tr("Group"),NULL,50);
+  appendHeader(tr("Attributes"),NULL,100);
 #ifndef WIN32
-  appendHeader("Link",NULL,200);
+  appendHeader(tr("Link"),NULL,200);
 #endif
   big_folder=new FXGIFIcon(getApp(),bigfolder);
   mini_folder=new FXGIFIcon(getApp(),minifolder);
@@ -320,7 +337,7 @@ long FXFileList::onDNDMotion(FXObject* sender,FXSelector sel,void* ptr){
       }
 
     // See if dropdirectory is writable
-    if(FXFile::isWritable(dropdirectory)){
+    if(FXStat::isWritable(dropdirectory)){
       FXTRACE((100,"accepting drop on %s\n",dropdirectory.text()));
       acceptDrop(DRAG_ACCEPT);
       }
@@ -332,7 +349,8 @@ long FXFileList::onDNDMotion(FXObject* sender,FXSelector sel,void* ptr){
 
 // Handle drag-and-drop drop
 long FXFileList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr){
-  FXuchar *data; FXuint len;
+  FXString dropfiles,filesrc,filedst,url;
+  FXint beg,end;
 
   // Cancel open up timer
   getApp()->removeTimeout(this,ID_OPENTIMER);
@@ -347,34 +365,38 @@ long FXFileList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr){
   if(FXIconList::onDNDDrop(sender,sel,ptr)) return 1;
 
   // Get uri-list of files being dropped
-  if(getDNDData(FROM_DRAGNDROP,urilistType,data,len)){
-    FXRESIZE(&data,FXuchar,len+1); data[len]='\0';
-    FXchar *p,*q;
-    p=q=(FXchar*)data;
-    while(*p){
-      while(*q && *q!='\r') q++;
-      FXString url(p,q-p);
-      FXString filesrc(FXURL::fileFromURL(url));
-      FXString filedst(dropdirectory+PATHSEPSTRING+FXFile::name(filesrc));
+  if(getDNDData(FROM_DRAGNDROP,urilistType,dropfiles)){
+
+    // Tell drag source we got it
+    dropFinished(DRAG_ACCEPT);
+
+    // Loop over urls
+    for(beg=0; beg<dropfiles.length(); beg=end+2){
+      if((end=dropfiles.find_first_of("\r\n",beg))<0) end=dropfiles.length();
+
+      // File url
+      url=dropfiles.mid(beg,end-beg);
+
+      // Source filename
+      filesrc=FXURL::fileFromURL(url);
+
+      // Destination filename
+      filedst=dropdirectory+PATHSEPSTRING+FXPath::name(filesrc);
 
       // Move, Copy, or Link as appropriate
       if(dropaction==DRAG_MOVE){
         FXTRACE((100,"Moving file: %s to %s\n",filesrc.text(),filedst.text()));
-        if(!FXFile::move(filesrc,filedst)) getApp()->beep();
+        if(!FXFile::moveFiles(filesrc,filedst)) getApp()->beep();
         }
       else if(dropaction==DRAG_COPY){
         FXTRACE((100,"Copying file: %s to %s\n",filesrc.text(),filedst.text()));
-        if(!FXFile::copy(filesrc,filedst)) getApp()->beep();
+        if(!FXFile::copyFiles(filesrc,filedst)) getApp()->beep();
         }
       else if(dropaction==DRAG_LINK){
         FXTRACE((100,"Linking file: %s to %s\n",filesrc.text(),filedst.text()));
         if(!FXFile::symlink(filesrc,filedst)) getApp()->beep();
         }
-      if(*q=='\r') q+=2;
-      p=q;
       }
-
-    FXFREE(&data);
     return 1;
     }
 
@@ -385,18 +407,13 @@ long FXFileList::onDNDDrop(FXObject* sender,FXSelector sel,void* ptr){
 // Somebody wants our dragged data
 long FXFileList::onDNDRequest(FXObject* sender,FXSelector sel,void* ptr){
   FXEvent *event=(FXEvent*)ptr;
-  FXuchar *data; FXuint len;
 
   // Perhaps the target wants to supply its own data
   if(FXIconList::onDNDRequest(sender,sel,ptr)) return 1;
 
   // Return list of filenames as a uri-list
   if(event->target==urilistType){
-    if(!dragfiles.empty()){
-      len=dragfiles.length();
-      FXMEMDUP(&data,dragfiles.text(),FXuchar,len);
-      setDNDData(FROM_DRAGNDROP,event->target,data,len);
-      }
+    setDNDData(FROM_DRAGNDROP,event->target,dragfiles);
     return 1;
     }
 
@@ -546,14 +563,14 @@ long FXFileList::onUpdToggleImages(FXObject* sender,FXSelector,void*){
 
 // Move up one level
 long FXFileList::onCmdDirectoryUp(FXObject*,FXSelector,void*){
-  setDirectory(FXFile::upLevel(directory));
+  setDirectory(FXPath::upLevel(directory));
   return 1;
   }
 
 
 // Determine if we can still go up more
-long FXFileList::onUpdDirectoryUp(FXObject* sender,FXSelector,void* ptr){
-  sender->handle(this,FXFile::isTopDirectory(directory)?FXSEL(SEL_COMMAND,ID_DISABLE):FXSEL(SEL_COMMAND,ID_ENABLE),ptr);
+long FXFileList::onUpdDirectoryUp(FXObject* sender,FXSelector,void*){
+  sender->handle(this,FXPath::isTopDirectory(directory)?FXSEL(SEL_COMMAND,ID_DISABLE):FXSEL(SEL_COMMAND,ID_ENABLE),NULL);
   return 1;
   }
 
@@ -727,8 +744,9 @@ long FXFileList::onCmdSortCase(FXObject*,FXSelector,void*){
 
 
 // Check if case sensitive
-long FXFileList::onUpdSortCase(FXObject* sender,FXSelector,void* ptr){
-  sender->handle(this,(sortfunc==ascendingCase || sortfunc==descendingCase) ? FXSEL(SEL_COMMAND,ID_CHECK) : FXSEL(SEL_COMMAND,ID_UNCHECK),ptr);
+long FXFileList::onUpdSortCase(FXObject* sender,FXSelector,void*){
+  sender->handle(this,(sortfunc==ascendingCase || sortfunc==descendingCase) ? FXSEL(SEL_COMMAND,ID_CHECK) : FXSEL(SEL_COMMAND,ID_UNCHECK),NULL);
+  sender->handle(this,(sortfunc==ascendingCase || sortfunc==ascending || sortfunc==descendingCase || sortfunc==descending) ? FXSEL(SEL_COMMAND,ID_ENABLE) : FXSEL(SEL_COMMAND,ID_DISABLE),NULL);
   return 1;
   }
 
@@ -753,167 +771,127 @@ long FXFileList::onUpdHeader(FXObject*,FXSelector,void*){
 
 
 // Compare file names
-FXint FXFileList::ascending(const FXIconItem* pa,const FXIconItem* pb){
-  register const FXFileItem *a=(FXFileItem*)pa;
-  register const FXFileItem *b=(FXFileItem*)pb;
-  register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
-  if(diff) return diff;
-  register const unsigned char *p=(const unsigned char*)a->label.text();
-  register const unsigned char *q=(const unsigned char*)b->label.text();
-  while(1){
-    if(*p > *q) return 1;
-    if(*p < *q) return -1;
-    if(*p<='\t') break;
-    p++;
-    q++;
+FXint FXFileList::ascending(const FXIconItem* a,const FXIconItem* b){
+  register FXint diff=static_cast<const FXFileItem*>(b)->isDirectory() - static_cast<const FXFileItem*>(a)->isDirectory();
+  if(diff==0){
+    return compareSection(a->label.text(),b->label.text(),0);
     }
-  return 0;
+  return diff;
   }
 
 
 // Compare file names, case insensitive
-FXint FXFileList::ascendingCase(const FXIconItem* pa,const FXIconItem* pb){
-  register const FXFileItem *a=(FXFileItem*)pa;
-  register const FXFileItem *b=(FXFileItem*)pb;
-  register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
-  if(diff) return diff;
-  register const unsigned char *p=(const unsigned char*)a->label.text();
-  register const unsigned char *q=(const unsigned char*)b->label.text();
-  while(1){
-    if(tolower(*p) > tolower(*q)) return 1;
-    if(tolower(*p) < tolower(*q)) return -1;
-    if(*p<='\t') break;
-    p++;
-    q++;
+FXint FXFileList::ascendingCase(const FXIconItem* a,const FXIconItem* b){
+  register FXint diff=static_cast<const FXFileItem*>(b)->isDirectory() - static_cast<const FXFileItem*>(a)->isDirectory();
+  if(diff==0){
+    return compareSectionCase(a->label.text(),b->label.text(),0);
     }
-  return 0;
+  return diff;
   }
 
 
 // Compare file types
-FXint FXFileList::ascendingType(const FXIconItem* pa,const FXIconItem* pb){
-  register const FXFileItem *a=(FXFileItem*)pa;
-  register const FXFileItem *b=(FXFileItem*)pb;
-  register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
-  if(diff) return diff;
-  register const unsigned char *p=(const unsigned char*)strchr(a->label.text(),'\t')+1;
-  register const unsigned char *q=(const unsigned char*)strchr(b->label.text(),'\t')+1;
-  while(1){
-    if(*p > *q) return 1;
-    if(*p < *q) return -1;
-    if(*p<='\t') break;
-    p++;
-    q++;
+FXint FXFileList::ascendingType(const FXIconItem* a,const FXIconItem* b){
+  register FXint diff=static_cast<const FXFileItem*>(b)->isDirectory() - static_cast<const FXFileItem*>(a)->isDirectory();
+  if(diff==0){
+    diff=compareSection(a->label.text(),b->label.text(),1);
+    if(diff==0){
+      return compareSectionCase(a->label.text(),b->label.text(),0);
+      }
     }
-  return FXIconList::ascendingCase(pa,pb);
+  return diff;
   }
 
 
 // Compare file size
-FXint FXFileList::ascendingSize(const FXIconItem* pa,const FXIconItem* pb){
-  register const FXFileItem *a=(FXFileItem*)pa;
-  register const FXFileItem *b=(FXFileItem*)pb;
-  register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
-  if(diff) return diff;
-  if(a->size>b->size) return 1;
-  if(a->size<b->size) return -1;
-  return FXIconList::ascendingCase(pa,pb);
+FXint FXFileList::ascendingSize(const FXIconItem* a,const FXIconItem* b){
+  register FXint diff=static_cast<const FXFileItem*>(b)->isDirectory() - static_cast<const FXFileItem*>(a)->isDirectory();
+  if(diff==0){
+    if(static_cast<const FXFileItem*>(a)->size > static_cast<const FXFileItem*>(b)->size) return 1;
+    if(static_cast<const FXFileItem*>(a)->size < static_cast<const FXFileItem*>(b)->size) return -1;
+    return compareSectionCase(a->label.text(),b->label.text(),0);
+    }
+  return diff;
   }
 
 
 // Compare file time
-FXint FXFileList::ascendingTime(const FXIconItem* pa,const FXIconItem* pb){
-  register const FXFileItem *a=(FXFileItem*)pa;
-  register const FXFileItem *b=(FXFileItem*)pb;
-  register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
-  if(diff) return diff;
-  if(a->date>b->date) return 1;
-  if(a->date<b->date) return -1;
-  return FXIconList::ascendingCase(pa, pb);
+FXint FXFileList::ascendingTime(const FXIconItem* a,const FXIconItem* b){
+  register FXint diff=(FXint)((FXFileItem*)b)->isDirectory() - (FXint)((FXFileItem*)a)->isDirectory();
+  if(diff==0){
+    if(static_cast<const FXFileItem*>(a)->date > static_cast<const FXFileItem*>(b)->date) return 1;
+    if(static_cast<const FXFileItem*>(a)->date < static_cast<const FXFileItem*>(b)->date) return -1;
+    return compareSectionCase(a->label.text(),b->label.text(),0);
+    }
+  return diff;
   }
 
 
 // Compare file user
-FXint FXFileList::ascendingUser(const FXIconItem* pa,const FXIconItem* pb){
-  register const FXFileItem *a=(FXFileItem*)pa;
-  register const FXFileItem *b=(FXFileItem*)pb;
-  register const unsigned char *p,*q;
-  register int i;
-  FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
-  if(diff) return diff;
-  for(p=(const unsigned char*)a->label.text(),i=4; *p && i; i-=(*p++=='\t'));
-  for(q=(const unsigned char*)b->label.text(),i=4; *q && i; i-=(*q++=='\t'));
-  while(1){
-    if(*p > *q) return 1;
-    if(*p < *q) return -1;
-    if(*p<='\t') break;
-    p++;
-    q++;
+FXint FXFileList::ascendingUser(const FXIconItem* a,const FXIconItem* b){
+  register FXint diff=static_cast<const FXFileItem*>(b)->isDirectory() - static_cast<const FXFileItem*>(a)->isDirectory();
+  if(diff==0){
+    diff=compareSection(a->label.text(),b->label.text(),4);
+    if(diff==0){
+      return compareSectionCase(a->label.text(),b->label.text(),0);
+      }
     }
-  return FXIconList::ascendingCase(pa,pb);
+  return diff;
   }
 
 
 // Compare file group
-FXint FXFileList::ascendingGroup(const FXIconItem* pa,const FXIconItem* pb){
-  register const FXFileItem *a=(FXFileItem*)pa;
-  register const FXFileItem *b=(FXFileItem*)pb;
-  register const unsigned char *p,*q;
-  register int i;
-  register FXint diff=(FXint)b->isDirectory() - (FXint)a->isDirectory();
-  if(diff) return diff;
-  for(p=(const unsigned char*)a->label.text(),i=5; *p && i; i-=(*p++=='\t'));
-  for(q=(const unsigned char*)b->label.text(),i=5; *q && i; i-=(*q++=='\t'));
-  while(1){
-    if(*p > *q) return 1;
-    if(*p < *q) return -1;
-    if(*p<='\t') break;
-    p++;
-    q++;
+FXint FXFileList::ascendingGroup(const FXIconItem* a,const FXIconItem* b){
+  register FXint diff=static_cast<const FXFileItem*>(b)->isDirectory() - static_cast<const FXFileItem*>(a)->isDirectory();
+  if(diff==0){
+    diff=compareSection(a->label.text(),b->label.text(),5);
+    if(diff==0){
+      return compareSectionCase(a->label.text(),b->label.text(),0);
+      }
     }
-  return FXIconList::ascendingCase(pa,pb);
+  return diff;
   }
 
 
 // Reversed compare file name, case insensitive
-FXint FXFileList::descendingCase(const FXIconItem* pa,const FXIconItem* pb){
-  return -FXFileList::ascendingCase(pa,pb);
+FXint FXFileList::descendingCase(const FXIconItem* a,const FXIconItem* b){
+  return -FXFileList::ascendingCase(a,b);
   }
 
 
 // Reversed compare file name
-FXint FXFileList::descending(const FXIconItem* pa,const FXIconItem* pb){
-  return -FXFileList::ascending(pa,pb);
+FXint FXFileList::descending(const FXIconItem* a,const FXIconItem* b){
+  return -FXFileList::ascending(a,b);
   }
 
 
 // Reversed compare file type
-FXint FXFileList::descendingType(const FXIconItem* pa,const FXIconItem* pb){
-  return -FXFileList::ascendingType(pa,pb);
+FXint FXFileList::descendingType(const FXIconItem* a,const FXIconItem* b){
+  return -FXFileList::ascendingType(a,b);
   }
 
 
 // Reversed compare file size
-FXint FXFileList::descendingSize(const FXIconItem* pa,const FXIconItem* pb){
-  return -FXFileList::ascendingSize(pa,pb);
+FXint FXFileList::descendingSize(const FXIconItem* a,const FXIconItem* b){
+  return -FXFileList::ascendingSize(a,b);
   }
 
 
 // Reversed compare file time
-FXint FXFileList::descendingTime(const FXIconItem* pa,const FXIconItem* pb){
-  return -FXFileList::ascendingTime(pa,pb);
+FXint FXFileList::descendingTime(const FXIconItem* a,const FXIconItem* b){
+  return -FXFileList::ascendingTime(a,b);
   }
 
 
 // Reversed compare file user
-FXint FXFileList::descendingUser(const FXIconItem* pa,const FXIconItem* pb){
-  return -FXFileList::ascendingUser(pa,pb);
+FXint FXFileList::descendingUser(const FXIconItem* a,const FXIconItem* b){
+  return -FXFileList::ascendingUser(a,b);
   }
 
 
 // Reversed compare file group
-FXint FXFileList::descendingGroup(const FXIconItem* pa,const FXIconItem* pb){
-  return -FXFileList::ascendingGroup(pa,pb);
+FXint FXFileList::descendingGroup(const FXIconItem* a,const FXIconItem* b){
+  return -FXFileList::ascendingGroup(a,b);
   }
 
 
@@ -948,13 +926,13 @@ long FXFileList::onCmdRefresh(FXObject*,FXSelector,void*){
 
 // Scan items to see if listing is necessary
 void FXFileList::scan(FXbool force){
-  struct stat info;
+  FXStat info;
 
   // Stat the current directory
-  if(FXFile::info(directory,info)){
+  if(FXStat::statFile(directory,info)){
 
     // New date of directory
-    FXTime newdate=(FXTime)FXMAX(info.st_mtime,info.st_ctime);
+    FXTime newdate=info.touched();
 
     // Forced, date was changed, or failed to get proper date and counter expired
     if(force || (timestamp!=newdate) || (counter==0)){
@@ -970,20 +948,24 @@ void FXFileList::scan(FXbool force){
 
   // Move to higher directory
   else{
-    setDirectory(FXFile::upLevel(directory));
+    setDirectory(FXPath::upLevel(directory));
     }
   }
 
 
 
 // Set current filename
-void FXFileList::setCurrentFile(const FXString& pathname){
+void FXFileList::setCurrentFile(const FXString& pathname,FXbool notify){
+  FXTRACE((100,"%s::setCurrentFile(%s)\n",getClassName(),pathname.text()));
   if(!pathname.empty()){
-    FXTRACE((100,"%s::setCurrentFile(%s)\n",getClassName(),pathname.text()));
-    setDirectory(FXFile::directory(pathname));
-    setCurrentItem(findItem(FXFile::name(pathname)));
-    setAnchorItem(getCurrentItem());
-    if(0<=getCurrentItem()) selectItem(getCurrentItem());
+    setDirectory(FXPath::directory(pathname));
+    FXint index=findItem(FXPath::name(pathname));
+    makeItemVisible(index);
+    setAnchorItem(index);
+    setCurrentItem(index,notify);
+    if(0<=index){
+      selectItem(index);
+      }
     }
   }
 
@@ -999,9 +981,9 @@ FXString FXFileList::getCurrentFile() const {
 void FXFileList::setDirectory(const FXString& pathname){
   if(!pathname.empty()){
     FXTRACE((100,"%s::setDirectory(%s)\n",getClassName(),pathname.text()));
-    FXString path=FXFile::absolute(directory,pathname);
-    while(!FXFile::isTopDirectory(path) && !(FXFile::isShare(path) || FXFile::isDirectory(path))){
-      path=FXFile::upLevel(path);
+    FXString path=FXPath::absolute(directory,pathname);
+    while(!FXPath::isTopDirectory(path) && !(FXPath::isShare(path) || FXStat::isDirectory(path))){
+      path=FXPath::upLevel(path);
       }
     if(directory!=path){
       directory=path;
@@ -1106,6 +1088,22 @@ void FXFileList::showOnlyFiles(FXbool shown){
   }
 
 
+// Return TRUE if showing parent directories
+FXbool FXFileList::showParents() const {
+  return (options&FILELIST_NO_PARENT)==0;
+  }
+
+
+// Show parent directories
+void FXFileList::showParents(FXbool shown) {
+  FXuint opts=shown ? (options&~FILELIST_NO_PARENT) : (options|FILELIST_NO_PARENT);
+  if(opts!=options){
+    options=opts;
+    scan(TRUE);
+    }
+  }
+
+
 // Compare till '\t' or '\0'
 static FXbool fileequal(const FXString& a,const FXString& b){
   register const FXuchar *p1=(const FXuchar *)a.text();
@@ -1126,13 +1124,6 @@ FXIconItem *FXFileList::createItem(const FXString& text,FXIcon *big,FXIcon* mini
   }
 
 
-/********************************************************************************
-*                                    X-Windows                                  *
-********************************************************************************/
-
-#ifndef WIN32
-
-
 // List directory
 void FXFileList::listItems(FXbool force){
   FXFileItem *oldlist=list;     // Old insert-order list
@@ -1153,9 +1144,8 @@ void FXFileList::listItems(FXbool force){
   FXString linkname;
   FXbool islink;
   FXbool istop;
-  struct stat info;
-  struct dirent *dp;
-  DIR *dirp;
+  FXStat info;
+  FXDir  dir;
 
   // Remember current item
   if(0<=current){ curitem=(FXFileItem*)items[current]; }
@@ -1164,20 +1154,19 @@ void FXFileList::listItems(FXbool force){
   items.clear();
 
   // Are we at the top directory?
-  istop=FXFile::isTopDirectory(directory);
+  istop=FXPath::isTopDirectory(directory);
+
+  // Assume not a link
+  islink=FALSE;
 
   // Get directory stream pointer
-  dirp=opendir(directory.text());
-  if(dirp){
+  if(dir.open(directory)){
 
     // Loop over directory entries
-#ifdef FOX_THREAD_SAFE
-    struct fxdirent dirresult;
-    while(!readdir_r(dirp,&dirresult,&dp) && dp){
-#else
-    while((dp=readdir(dirp))!=NULL){
-#endif
-      name=dp->d_name;
+    while(dir.next()){
+
+      // Get file name
+      name=dir.name();
 
       // Hidden files of the form ".xxx" are normally not shown, but ".." is so we can
       // navigate up as well as down.  However, at the root level we can't go up any
@@ -1185,10 +1174,10 @@ void FXFileList::listItems(FXbool force){
       // as a directory when we're in directory selection mode.
       if(name[0]=='.'){
         if(name[1]==0){
-          if(!istop) continue;
+          if((options&FILELIST_NO_PARENT) || !istop) continue;
           }
         else if(name[1]=='.' && name[2]==0){
-          if(istop) continue;
+          if((options&FILELIST_NO_PARENT) || istop) continue;
           }
         else{
           if(!(options&FILELIST_SHOWHIDDEN)) continue;
@@ -1200,18 +1189,31 @@ void FXFileList::listItems(FXbool force){
       if(!ISPATHSEP(pathname[pathname.length()-1])) pathname+=PATHSEPSTRING;
       pathname+=name;
 
+#ifdef WIN32
+
       // Get file/link info
-      if(!FXFile::linkinfo(pathname,info)) continue;
+      if(!FXStat::statFile(pathname,info)) continue;
+
+      // Hidden file or directory normally not shown
+      if(info.isHidden() && !(options&FILELIST_SHOWHIDDEN)) continue;
+
+#else
+
+      // Get file/link info
+      if(!FXStat::statLink(pathname,info)) continue;
 
       // If its a link, get the info on file itself
-      islink=S_ISLNK(info.st_mode);
-      if(islink && !FXFile::info(pathname,info)) continue;
+      islink=info.isLink();
+      if(islink && !FXStat::statFile(pathname,info)) continue;
+
+#endif
 
       // If it is a file and we want only directories or doesn't match, skip it
-      if(!S_ISDIR(info.st_mode) && ((options&FILELIST_SHOWDIRS) || !FXFile::match(pattern,name,matchmode))) continue;
+      if(!info.isDirectory() && ((options&FILELIST_SHOWDIRS) || !FXPath::match(pattern,name,matchmode))) continue;
 
       // If it is a directory and we want only files, skip it
-      if(S_ISDIR(info.st_mode) && (options&FILELIST_SHOWFILES)) continue;
+      if(info.isDirectory() && (options&FILELIST_SHOWFILES)) continue;
+
 
       // Find it, and take it out from the old list if found
       for(pp=po; (item=*pp)!=NULL; pp=&item->link){
@@ -1224,7 +1226,7 @@ void FXFileList::listItems(FXbool force){
         }
 
       // Make new item if we have to
-      item=(FXFileItem*)createItem(NULL,NULL,NULL,NULL);
+      item=(FXFileItem*)createItem(FXString::null,NULL,NULL,NULL);
 
       // Add to insert-order list
 fnd:  *pn=item;
@@ -1235,59 +1237,59 @@ fnd:  *pn=item;
       items.append(item);
 
       // Update only if something changed
-      if(force || item->label.empty() || item->date!=info.st_mtime || item->size!=info.st_size){
+      if(force || item->label.empty() || item->date!=info.modified() || item->size!=info.size()){
 
         // Obtain user name
-        usrid=FXFile::owner(info.st_uid);
+        usrid=FXSystem::userName(info.user());
 
         // Obtain group name
-        grpid=FXFile::group(info.st_gid);
+        grpid=FXSystem::groupName(info.group());
 
         // Permissions
-        atts=FXFile::permissions(info.st_mode);
+        atts=FXSystem::modeString(info.mode());
 
         // Mod time
-        mod=FXFile::time(info.st_mtime);
+        mod=FXSystem::time(info.modified());
 
         // Link
         if(islink) linkname=FXFile::symlink(pathname); else linkname=FXString::null;
 
         // Update flags
-        if(info.st_mode&(S_IXUSR|S_IXGRP|S_IXOTH)){item->state|=FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::EXECUTABLE;}
-        if(S_ISDIR(info.st_mode)){item->state|=FXFileItem::FOLDER;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::FOLDER;}
-        if(S_ISCHR(info.st_mode)){item->state|=FXFileItem::CHARDEV;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::CHARDEV;}
-        if(S_ISBLK(info.st_mode)){item->state|=FXFileItem::BLOCKDEV;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::BLOCKDEV;}
-        if(S_ISFIFO(info.st_mode)){item->state|=FXFileItem::FIFO;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::FIFO;}
-        if(S_ISSOCK(info.st_mode)){item->state|=FXFileItem::SOCK;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::SOCK;}
+        if(info.isExecutable()){item->state|=FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::EXECUTABLE;}
+        if(info.isDirectory()){item->state|=FXFileItem::FOLDER;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::FOLDER;}
+        if(info.isCharacter()){item->state|=FXFileItem::CHARDEV;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::CHARDEV;}
+        if(info.isBlock()){item->state|=FXFileItem::BLOCKDEV;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::BLOCKDEV;}
+        if(info.isFifo()){item->state|=FXFileItem::FIFO;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::FIFO;}
+        if(info.isSocket()){item->state|=FXFileItem::SOCK;item->state&=~FXFileItem::EXECUTABLE;}else{item->state&=~FXFileItem::SOCK;}
         if(islink){item->state|=FXFileItem::SYMLINK;}else{item->state&=~FXFileItem::SYMLINK;}
 
         // We can drag items
-        item->state|=FXFileItem::DRAGGABLE;
+        item->setDraggable(TRUE);
 
         // File size
-        item->size=info.st_size;
+        item->size=info.size();
 
         // File access time
-        item->date=info.st_mtime;
+        item->date=info.modified();
 
         // No associations
         item->assoc=NULL;
 
         // Determine icons and type
-        if(item->state&FXFileItem::FOLDER){
+        if(item->isDirectory()){
           extension="File Folder";
           item->setBigIcon(big_folder);
           item->setMiniIcon(mini_folder);
           if(associations) item->assoc=associations->findDirBinding(pathname.text());
           }
-        else if(item->state&FXFileItem::EXECUTABLE){
+        else if(item->isExecutable()){
           extension="Application";
           item->setBigIcon(big_app);
           item->setMiniIcon(mini_app);
           if(associations) item->assoc=associations->findExecBinding(pathname.text());
           }
         else{
-          extension=FXFile::extension(pathname).upper();
+          extension=FXPath::extension(pathname).upper();
           if(!extension.empty()) extension+=" File";
           item->setBigIcon(big_doc);
           item->setMiniIcon(mini_doc);
@@ -1304,250 +1306,26 @@ fnd:  *pn=item;
         // Attempt to load thumbnail
         if(associations && (options&FILELIST_SHOWIMAGES)){
           source=associations->getIconDict()->getIconSource();
-          icon=source->loadScaledIcon(pathname,imagesize);
+          icon=source->loadScaledIconFile(pathname,imagesize);
           if(icon) item->setBigIcon(icon,TRUE);
           }
 
         // Update item information
-        item->label.format("%s\t%s\t%llu\t%s\t%s\t%s\t%s\t%s",name.text(),extension.text(),item->size,mod.text(),usrid.text(),grpid.text(),atts.text(),linkname.text());
-
-        // Create item
-        if(id()) item->create();
-        }
-      }
-    closedir(dirp);
-    }
-
-  // Wipe items remaining in list:- they have disappeared!!
-  for(item=oldlist; item; item=link){
-    link=item->link;
-    delete item;
-    }
-
-  // Validate
-  if(current>=items.no()) current=-1;
-  if(anchor>=items.no()) anchor=-1;
-  if(extent>=items.no()) extent=-1;
-
-  // Remember new list
-  list=newlist;
-
-  // Gotta recalc size of content
-  recalc();
-  }
-
-
-/********************************************************************************
-*                                   MS-Windows                                  *
-********************************************************************************/
-
+#ifndef WIN32
+#if defined(__LP64__) || defined(_LP64) || (_MIPS_SZLONG == 64) || (__WORDSIZE == 64)
+        item->label.format("%s\t%s\t%ld\t%s\t%s\t%s\t%s\t%s",name.text(),extension.text(),item->size,mod.text(),usrid.text(),grpid.text(),atts.text(),linkname.text());
 #else
-
-
-// Convert FILETIME (# 100ns since 01/01/1601) to time_t (# s since 01/01/1970)
-static time_t fxfiletime(const FILETIME& ft){
-  FXlong ll=(((FXlong)ft.dwHighDateTime)<<32)+((FXlong)ft.dwLowDateTime);
-#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(__SC__)
-  ll=ll-116444736000000000LL;
-#else
-  ll=ll-116444736000000000L;
+        item->label.format("%s\t%s\t%lld\t%s\t%s\t%s\t%s\t%s",name.text(),extension.text(),item->size,mod.text(),usrid.text(),grpid.text(),atts.text(),linkname.text());
 #endif
-  ll=ll/10000000;
-  return (time_t)ll;
-  }
-
-
-// List directory
-void FXFileList::listItems(FXbool force){
-  FXFileItem *oldlist=list;     // Old insert-order list
-  FXFileItem *newlist=NULL;     // New insert-order list
-  FXFileItem **po=&oldlist;     // Head of old list
-  FXFileItem **pn=&newlist;     // Head of new list
-  FXFileItem *curitem=NULL;
-  FXFileItem *item,*link,**pp;
-  FXIconSource *source;
-  FXIcon *icon;
-  FXString pathname;
-  FXString extension;
-  FXString name;
-  FXString grpid;
-  FXString usrid;
-  FXString atts;
-  FXString mod;
-  FXlong filesize;
-  FXint istop;
-  time_t filetime;
-  WIN32_FIND_DATA ffData;
-  SHFILEINFO sfi;
-  HANDLE hFindFile;
-
-  // Remember current item
-  if(0<=current){ curitem=(FXFileItem*)items[current]; }
-
-  // Start inserting
-  items.clear();
-
-  // Are we at the top directory?
-  istop=FXFile::isTopDirectory(directory);
-
-  // Set path to stat with
-  pathname=directory;
-  if(!ISPATHSEP(pathname[pathname.length()-1])) pathname+=PATHSEPSTRING;
-  pathname+="*";
-
-  // Get file find handle and first file's info
-  hFindFile=FindFirstFile(pathname.text(),&ffData);
-  if(hFindFile!=INVALID_HANDLE_VALUE){
-
-    // Loop over directory entries
-    do{
-      name=ffData.cFileName;
-
-      // Normally ".." is shown so we can navigate up as well as down.  However, at the top
-      // level we can't go up any further so we show "." but not ".."; this allows us to
-      // explicitly select "\." as a directory when we're in directory selection mode.
-      if(name[0]=='.'){
-        if(name[1]==0){
-          if(!istop) continue;
-          }
-        else if(name[1]=='.' && name[2]==0){
-          if(istop) continue;
-          }
-        }
-
-      // Hidden file or directory normally not shown
-      if((ffData.dwFileAttributes&FILE_ATTRIBUTE_HIDDEN) && !(options&FILELIST_SHOWHIDDEN)) continue;
-
-      // If it is a file and we want only directories or doesn't match, skip it
-      if(!(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) && ((options&FILELIST_SHOWDIRS) || !FXFile::match(pattern,name,matchmode))) continue;
-
-      // If it is a directory and we want only files, skip it
-      if((ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) && (options&FILELIST_SHOWFILES)) continue;
-
-      // Build full pathname
-      pathname=directory;
-      if(!ISPATHSEP(pathname[pathname.length()-1])) pathname+=PATHSEPSTRING;
-      pathname+=name;
-
-      // Convert it
-      filetime=fxfiletime(ffData.ftLastWriteTime);
-      filesize=(((FXlong)ffData.nFileSizeHigh)<<32)+((FXlong)ffData.nFileSizeLow);
-
-      // Find it, and take it out from the old list if found
-      for(pp=po; (item=*pp)!=NULL; pp=&item->link){
-        if(fileequal(item->label,name)){
-          *pp=item->link;
-          item->link=NULL;
-          po=pp;
-          goto fnd;
-          }
-        }
-
-      // Make new item if we have to
-      item=(FXFileItem*)createItem(NULL,NULL,NULL,NULL);
-
-      // Add to insert-order list
-fnd:  *pn=item;
-      pn=&item->link;
-
-      // Append
-      if(item==curitem) current=items.no();
-      items.append(item);
-
-
-      // Update only if something changed
-      if(force || item->label.empty() || item->date!=filetime || item->size!=filesize){
-
-        // Obtain user name (no Win95 equivalent?)
-        usrid=FXFile::owner(0);
-
-        // Obtain group name (no Win95 equivalent?)
-        grpid=FXFile::group(0);
-
-        // Permissions
-        atts=FXFile::permissions(0666);
-
-        // Mod time
-        mod=FXFile::time(filetime);
-
-        // Is it a directory?
-        if(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY){
-          atts[0]='d';
-          }
-
-        // Check if file is read-only
-        if(ffData.dwFileAttributes&FILE_ATTRIBUTE_READONLY){
-          atts[2]='-'; atts[5]='-'; atts[8]='-';
-          }
-
-        // Is it an executable file?
-        if(SHGetFileInfo(pathname.text(),0,&sfi,sizeof(SHFILEINFO),SHGFI_EXETYPE)==0){
-          atts[3]='-'; atts[6]='-'; atts[9]='-';
-          item->state&=~FXFileItem::EXECUTABLE;
-          }
-        else{
-          item->state|=FXFileItem::EXECUTABLE;
-          }
-
-        // Flags
-        if(ffData.dwFileAttributes&FILE_ATTRIBUTE_DIRECTORY) item->state|=FXFileItem::FOLDER; else item->state&=~FXFileItem::FOLDER;
-
-        // We can drag items
-        item->state|=FXFileItem::DRAGGABLE;
-
-        // File size
-        item->size=filesize;
-
-        // File access time
-        item->date=filetime;
-
-        // No associations
-        item->assoc=NULL;
-
-        // Determine icons and type
-        if(item->state&FXFileItem::FOLDER){
-          extension="File Folder";
-          item->setBigIcon(big_folder);
-          item->setMiniIcon(mini_folder);
-          if(associations) item->assoc=associations->findDirBinding(pathname.text());
-          }
-        else if(item->state&FXFileItem::EXECUTABLE){
-          extension="Application";
-          item->setBigIcon(big_app);
-          item->setMiniIcon(mini_app);
-          if(associations) item->assoc=associations->findExecBinding(pathname.text());
-          }
-        else{
-          extension=FXFile::extension(pathname).upper();
-          if(!extension.empty()) extension+=" File";
-          item->setBigIcon(big_doc);
-          item->setMiniIcon(mini_doc);
-          if(associations) item->assoc=associations->findFileBinding(pathname.text());
-          }
-
-        // If association is found, use it
-        if(item->assoc){
-          extension=item->assoc->extension;
-          if(item->assoc->bigicon) item->setBigIcon(item->assoc->bigicon);
-          if(item->assoc->miniicon) item->setMiniIcon(item->assoc->miniicon);
-          }
-
-        // Attempt to load thumbnail
-        if(associations && (options&FILELIST_SHOWIMAGES)){
-          source=associations->getIconDict()->getIconSource();
-          icon=source->loadScaledIcon(pathname,imagesize);
-          if(icon) item->setBigIcon(icon,TRUE);
-          }
-
-        // Update item information
+#else
         item->label.format("%s\t%s\t%I64u\t%s\t%s\t%s\t%s",name.text(),extension.text(),item->size,mod.text(),usrid.text(),grpid.text(),atts.text());
+#endif
 
         // Create item
         if(id()) item->create();
         }
       }
-    while(FindNextFile(hFindFile,&ffData));
-    FindClose(hFindFile);
+    dir.close();
     }
 
   // Wipe items remaining in list:- they have disappeared!!
@@ -1569,9 +1347,6 @@ fnd:  *pn=item;
   }
 
 
-#endif
-
-
 /*******************************************************************************/
 
 
@@ -1613,7 +1388,7 @@ FXString FXFileList::getItemFilename(FXint index) const {
 // Get full pathname to item
 FXString FXFileList::getItemPathname(FXint index) const {
   if(index<0 || items.no()<=index){ fxerror("%s::getItemPathname: index out of range.\n",getClassName()); }
-  return FXFile::absolute(directory,items[index]->label.section('\t',0));
+  return FXPath::absolute(directory,items[index]->label.section('\t',0));
   }
 
 
