@@ -3,7 +3,7 @@
 *                 M u l i t h r e a d i n g   S u p p o r t                     *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2004,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2004,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,16 +19,27 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXThread.cpp,v 1.38.2.3 2005/11/09 10:30:46 fox Exp $                        *
+* $Id: FXThread.cpp,v 1.53.2.7 2006/07/28 05:30:49 fox Exp $                    *
 ********************************************************************************/
+#ifdef WIN32
+#if _WIN32_WINNT < 0x0400
+#define _WIN32_WINNT 0x0400
+#endif
+#endif
 #include "xincs.h"
 #include "fxver.h"
 #include "fxdefs.h"
 #include "FXThread.h"
-
 #ifndef WIN32
+#ifdef APPLE
+#include <mach/mach_init.h>
+#include <mach/semaphore.h>
+#include <mach/task.h>
+#include <pthread.h>
+#else
 #include <pthread.h>
 #include <semaphore.h>
+#endif
 #else
 #include <process.h>
 #endif
@@ -59,6 +70,12 @@
 
   - Picked unsigned long so as to ensure alignment issues are taken
     care off.
+
+  - I now believe its safe to set tid=0 after run returns; if FXThread
+    is destroyed then the execution is stopped immediately; if the thread
+    exits, tid is also set to 0.  If the thread is cancelled, tid is also
+    set to 0.  In no circumstance I can see is it possible for run() to
+    return when FXThread no longer exists.
 */
 
 using namespace FX;
@@ -123,6 +140,45 @@ FXMutex::~FXMutex(){
 /*******************************************************************************/
 
 
+#ifdef APPLE
+
+
+// Initialize semaphore
+FXSemaphore::FXSemaphore(FXint initial){
+  // If this fails on your machine, determine what value
+  // of sizeof(MPSemaphoreID*) is supposed to be on your
+  // machine and mail it to: jeroen@fox-toolkit.org!!
+  //FXTRACE((150,"sizeof(MPSemaphoreID*)=%d\n",sizeof(MPSemaphoreID*)));
+  FXASSERT(sizeof(data)>=sizeof(MPSemaphoreID*));
+  MPCreateSemaphore(2147483647,0,(MPSemaphoreID*)data);
+  }
+
+
+// Decrement semaphore
+void FXSemaphore::wait(){
+  MPWaitOnSemaphore(*((MPSemaphoreID*)data),kDurationForever);
+  }
+
+
+// Decrement semaphore but don't block
+FXbool FXSemaphore::trywait(){
+  return MPWaitOnSemaphore(*((MPSemaphoreID*)data),kDurationImmediate)==noErr;
+  }
+
+
+// Increment semaphore
+void FXSemaphore::post(){
+  MPSignalSemaphore(*((MPSemaphoreID*)data));
+  }
+
+
+// Delete semaphore
+FXSemaphore::~FXSemaphore(){
+  MPDeleteSemaphore(*((MPSemaphoreID*)data));
+  }
+
+#else
+
 // Initialize semaphore
 FXSemaphore::FXSemaphore(FXint initial){
   // If this fails on your machine, determine what value
@@ -157,6 +213,7 @@ FXSemaphore::~FXSemaphore(){
   sem_destroy((sem_t*)data);
   }
 
+#endif
 
 /*******************************************************************************/
 
@@ -191,14 +248,11 @@ void FXCondition::wait(FXMutex& mtx){
 
 
 // Wait for condition but fall through after timeout
-FXbool FXCondition::wait(FXMutex& mtx,FXuint ms){
+FXbool FXCondition::wait(FXMutex& mtx,FXlong nsec){
   register int result;
   struct timespec ts;
-  struct timeval tv;
-  gettimeofday(&tv,NULL);
-  ts.tv_nsec=((ms%1000)*1000+tv.tv_usec)*1000;
-  ts.tv_sec=(ms/1000)+(ts.tv_nsec/1000000000)+tv.tv_sec;
-  ts.tv_nsec=ts.tv_nsec%1000000000;
+  ts.tv_sec=nsec/1000000000;
+  ts.tv_nsec=nsec%1000000000;
 x:result=pthread_cond_timedwait((pthread_cond_t*)data,(pthread_mutex_t*)&mtx.data,&ts);
   if(result==EINTR) goto x;
   return result!=ETIMEDOUT;
@@ -217,11 +271,12 @@ FXCondition::~FXCondition(){
 static pthread_key_t self_key;
 
 // Global initializer for the self_key variable
-struct TLSKEYINIT { 
-  TLSKEYINIT(){ pthread_key_create(&self_key,NULL); } 
+struct TLSKEYINIT {
+  TLSKEYINIT(){ pthread_key_create(&self_key,NULL); }
  ~TLSKEYINIT(){ pthread_key_delete(self_key); }
   };
 
+
 // Extern declaration prevents overzealous optimizer from noticing we're
 // never using this object, and subsequently eliminating it from the code.
 extern TLSKEYINIT initializer;
@@ -259,6 +314,7 @@ void* FXThread::execute(void* thread){
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
   pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);
   try{ code=((FXThread*)thread)->run(); } catch(...){ }
+  ((FXThread*)thread)->tid=0;
   return (void*)(FXival)code;
   }
 
@@ -281,8 +337,9 @@ FXbool FXThread::start(unsigned long stacksize){
 
 // Suspend calling thread until thread is done
 FXbool FXThread::join(FXint& code){
+  register pthread_t ttid=(pthread_t)tid;
   void *trc=NULL;
-  if(tid && pthread_join((pthread_t)tid,&trc)==0){
+  if(ttid && pthread_join(ttid,&trc)==0){
     code=(FXint)(FXival)trc;
     tid=0;
     return TRUE;
@@ -293,7 +350,8 @@ FXbool FXThread::join(FXint& code){
 
 // Suspend calling thread until thread is done
 FXbool FXThread::join(){
-  if(tid && pthread_join((pthread_t)tid,NULL)==0){
+  register pthread_t ttid=(pthread_t)tid;
+  if(ttid && pthread_join(ttid,NULL)==0){
     tid=0;
     return TRUE;
     }
@@ -303,8 +361,9 @@ FXbool FXThread::join(){
 
 // Cancel the thread
 FXbool FXThread::cancel(){
-  if(tid && pthread_cancel((pthread_t)tid)==0){
-    pthread_join((pthread_t)tid,NULL);
+  register pthread_t ttid=(pthread_t)tid;
+  if(ttid && pthread_cancel(ttid)==0){
+    pthread_join(ttid,NULL);
     tid=0;
     return TRUE;
     }
@@ -314,12 +373,14 @@ FXbool FXThread::cancel(){
 
 // Detach thread
 FXbool FXThread::detach(){
-  return tid && pthread_detach((pthread_t)tid)==0;
+  register pthread_t ttid=(pthread_t)tid;
+  return ttid && pthread_detach(ttid)==0;
   }
 
 
 // Exit calling thread
 void FXThread::exit(FXint code){
+  if(self()){ self()->tid=0; }
   pthread_exit((void*)(FXival)code);
   }
 
@@ -330,12 +391,69 @@ void FXThread::yield(){
   }
 
 
+// Get time in nanoseconds since Epoch
+FXlong FXThread::time(){
+#ifdef __USE_POSIX199309
+  const FXlong seconds=1000000000;
+  struct timespec ts;
+  clock_gettime(CLOCK_REALTIME,&ts);
+  return ts.tv_sec*seconds+ts.tv_nsec;
+#else
+  const FXlong seconds=1000000000;
+  const FXlong microseconds=1000;
+  struct timeval tv;
+  gettimeofday(&tv,NULL);
+  return tv.tv_sec*seconds+tv.tv_usec*microseconds;
+#endif
+  }
+
+
 // Sleep for some time
-void FXThread::sleep(unsigned long sec,unsigned long nsec){
+void FXThread::sleep(FXlong nsec){
+#ifdef __USE_POSIX199309
+  const FXlong seconds=1000000000;
   struct timespec value;
-  value.tv_sec=sec+nsec/1000000000;     // A few seconds is billions and billions of nanoseconds!
-  value.tv_nsec=nsec%1000000000;
+  value.tv_sec=nsec/seconds;
+  value.tv_nsec=nsec%seconds;
   nanosleep(&value,NULL);
+#else
+  const FXlong seconds=1000000000;
+  const FXlong microseconds=1000;
+  const FXlong milliseconds=1000000;
+  struct timeval value;
+  value.tv_usec=(nsec/microseconds)%milliseconds;
+  value.tv_sec=nsec/seconds;
+  select(1,0,0,0,&value);
+#endif
+  }
+
+
+// Wake at appointed time
+void FXThread::wakeat(FXlong nsec){
+#ifdef __USE_POSIX199309
+  const FXlong seconds=1000000000;
+  struct timespec value;
+#ifdef __USE_XOPEN2K
+  value.tv_sec=nsec/seconds;
+  value.tv_nsec=nsec%seconds;
+  clock_nanosleep(CLOCK_REALTIME,TIMER_ABSTIME,&value,NULL);
+#else
+  nsec-=FXThread::time();
+  if(nsec<0) nsec=0;
+  value.tv_sec=nsec/seconds;
+  value.tv_nsec=nsec%seconds;
+  nanosleep(&value,NULL);
+#endif
+#else
+  const FXlong seconds=1000000000;
+  const FXlong microseconds=1000;
+  const FXlong milliseconds=1000000;
+  struct timeval value;
+  if(nsec<0) nsec=0;
+  value.tv_usec=(nsec/microseconds)%milliseconds;
+  value.tv_sec=nsec/seconds;
+  select(1,0,0,0,&value);
+#endif
   }
 
 
@@ -353,23 +471,30 @@ FXThreadID FXThread::current(){
 
 // Set thread priority
 void FXThread::priority(FXint prio){
-  sched_param sched={0};
-  int priomin,priomax;
-  if(tid){
-    priomax=sched_get_priority_max(SCHED_OTHER);
-    priomin=sched_get_priority_min(SCHED_OTHER);
+  register pthread_t ttid=(pthread_t)tid;
+  if(ttid){
+    sched_param sched={0};
+    int pcy=0;
+    pthread_getschedparam(ttid,&pcy,&sched);
+#if defined(_POSIX_PRIORITY_SCHEDULING)
+    int priomax=sched_get_priority_max(pcy);
+    int priomin=sched_get_priority_min(pcy);
     sched.sched_priority=FXCLAMP(priomin,prio,priomax);
-    pthread_setschedparam((pthread_t)tid,SCHED_OTHER,&sched);
+#elif defined(PTHREAD_MINPRIORITY)
+    sched.sched_priority=FXCLAMP(PTHREAD_MIN_PRIORITY,prio,PTHREAD_MAX_PRIORITY);
+#endif
+    pthread_setschedparam(ttid,pcy,&sched);
     }
   }
 
 
 // Return thread priority
 FXint FXThread::priority(){
-  sched_param sched={0};
-  int policy;
-  if(tid){
-    pthread_getschedparam((pthread_t)tid,&policy,&sched);
+  register pthread_t ttid=(pthread_t)tid;
+  if(ttid){
+    sched_param sched={0};
+    int pcy=0;
+    pthread_getschedparam(ttid,&pcy,&sched);
     return sched.sched_priority;
     }
   return 0;
@@ -378,7 +503,10 @@ FXint FXThread::priority(){
 
 // Destroy; if it was running, stop it
 FXThread::~FXThread(){
-  if(tid){ pthread_cancel((pthread_t)tid); }
+  register pthread_t ttid=(pthread_t)tid;
+  if(ttid){
+    pthread_cancel(ttid);
+    }
   }
 
 
@@ -479,8 +607,8 @@ FXSemaphore::~FXSemaphore(){
 // See: (http://www.cs.wustl.edu/~schmidt/win32-cv-1.html).
 //
 // Our implementation however initializes the Event objects in
-// the constructor, under the assumption that you're not creating
-// condition object if you weren't planning to use them somewhere.
+// the constructor, under the assumption that you wouldn't be creating
+// a condition object if you weren't planning to use them somewhere.
 
 
 // Initialize condition
@@ -529,12 +657,13 @@ void FXCondition::wait(FXMutex& mtx){
 
 
 // Wait using single global mutex
-FXbool FXCondition::wait(FXMutex& mtx,FXuint ms){
+FXbool FXCondition::wait(FXMutex& mtx,FXlong nsec){
   EnterCriticalSection((CRITICAL_SECTION*)&data[3]);
   data[2]++;
   LeaveCriticalSection((CRITICAL_SECTION*)&data[3]);
   mtx.unlock();
-  DWORD result=WaitForMultipleObjects(2,(HANDLE*)data,0,ms);
+  nsec-=FXThread::time();
+  DWORD result=WaitForMultipleObjects(2,(HANDLE*)data,0,nsec/1000000);
   EnterCriticalSection((CRITICAL_SECTION*)&data[3]);
   data[2]--;
   int last_waiter=(result==WAIT_OBJECT_0+1)&&(data[2]==0);      // Unblocked by broadcast & no other blocked threads
@@ -599,6 +728,7 @@ unsigned int CALLBACK FXThread::execute(void* thread){
   register FXint code=-1;
   TlsSetValue(self_key,thread);
   try{ code=((FXThread*)thread)->run(); } catch(...){ }
+  ((FXThread*)thread)->tid=0;
   return code;
   }
 
@@ -613,9 +743,10 @@ FXbool FXThread::start(unsigned long stacksize){
 
 // Suspend calling thread until thread is done
 FXbool FXThread::join(FXint& code){
-  if(tid && WaitForSingleObject((HANDLE)tid,INFINITE)==WAIT_OBJECT_0){
-    GetExitCodeThread((HANDLE)tid,(DWORD*)&code);
-    CloseHandle((HANDLE)tid);
+  register HANDLE ttid=(HANDLE)tid;
+  if(ttid && WaitForSingleObject(ttid,INFINITE)==WAIT_OBJECT_0){
+    GetExitCodeThread(ttid,(DWORD*)&code);
+    CloseHandle(ttid);
     tid=0;
     return TRUE;
     }
@@ -625,8 +756,9 @@ FXbool FXThread::join(FXint& code){
 
 // Suspend calling thread until thread is done
 FXbool FXThread::join(){
-  if(tid && WaitForSingleObject((HANDLE)tid,INFINITE)==WAIT_OBJECT_0){
-    CloseHandle((HANDLE)tid);
+  register HANDLE ttid=(HANDLE)tid;
+  if(ttid && WaitForSingleObject(ttid,INFINITE)==WAIT_OBJECT_0){
+    CloseHandle(ttid);
     tid=0;
     return TRUE;
     }
@@ -636,8 +768,9 @@ FXbool FXThread::join(){
 
 // Cancel the thread
 FXbool FXThread::cancel(){
-  if(tid && TerminateThread((HANDLE)tid,0)){
-    CloseHandle((HANDLE)tid);
+  register HANDLE ttid=(HANDLE)tid;
+  if(ttid && TerminateThread(ttid,0)){
+    CloseHandle(ttid);
     tid=0;
     return TRUE;
     }
@@ -653,6 +786,7 @@ FXbool FXThread::detach(){
 
 // Exit calling thread
 void FXThread::exit(FXint code){
+  if(self()){ self()->tid=0; }
   ExitThread(code);
   }
 
@@ -663,9 +797,29 @@ void FXThread::yield(){
   }
 
 
+// Get time in nanoseconds since Epoch
+FXlong FXThread::time(){
+  FXlong now;
+  GetSystemTimeAsFileTime((FILETIME*)&now);
+#if defined(__CYGWIN__) || defined(__MINGW32__) || defined(__SC__)
+  return (now-116444736000000000LL)*100LL;
+#else
+  return (now-116444736000000000L)*100L;
+#endif
+  }
+
+
 // Sleep for some time
-void FXThread::sleep(unsigned long sec,unsigned long nsec){
-  Sleep(sec*1000+nsec/1000000);
+void FXThread::sleep(FXlong nsec){
+  Sleep(nsec/1000000);
+  }
+
+
+// Wake at appointed time
+void FXThread::wakeat(FXlong nsec){
+  nsec-=FXThread::time();
+  if(nsec<0) nsec=0;
+  Sleep(nsec/1000000);
   }
 
 
@@ -683,16 +837,18 @@ FXThread* FXThread::self(){
 
 // Set thread priority
 void FXThread::priority(FXint prio){
-  if(tid){
-    SetThreadPriority((HANDLE)tid,prio);
+  register HANDLE ttid=(HANDLE)tid;
+  if(ttid){
+    SetThreadPriority(ttid,prio);
     }
   }
 
 
 // Return thread priority
 FXint FXThread::priority(){
-  if(tid){
-    return GetThreadPriority((HANDLE)tid);
+  register HANDLE ttid=(HANDLE)tid;
+  if(ttid){
+    return GetThreadPriority(ttid);
     }
   return 0;
   }
@@ -700,7 +856,11 @@ FXint FXThread::priority(){
 
 // Destroy
 FXThread::~FXThread(){
-  if(tid){ TerminateThread((HANDLE)tid,0); CloseHandle((HANDLE)tid); }
+  register HANDLE ttid=(HANDLE)tid;
+  if(ttid){
+    TerminateThread(ttid,0);
+    CloseHandle(ttid);
+    }
   }
 
 
