@@ -3,7 +3,7 @@
 *              P r i v a t e   I n t e r n a l   F u n c t i o n s              *
 *                                                                               *
 *********************************************************************************
-* Copyright (C) 2000,2005 by Jeroen van der Zijp.   All Rights Reserved.        *
+* Copyright (C) 2000,2006 by Jeroen van der Zijp.   All Rights Reserved.        *
 *********************************************************************************
 * This library is free software; you can redistribute it and/or                 *
 * modify it under the terms of the GNU Lesser General Public                    *
@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: fxpriv.h,v 1.30 2005/02/01 04:10:23 fox Exp $                            *
+* $Id: fxpriv.h,v 1.36.2.1 2006/03/28 13:02:54 fox Exp $                            *
 ********************************************************************************/
 
 namespace FX {
@@ -51,6 +51,8 @@ namespace FX {
 #define WM_DND_FINISH_PRIVATE   (WM_APP+20)
 #endif
 
+
+
 // Named color
 struct FXNamedColor {
   const FXchar *name;
@@ -77,16 +79,12 @@ extern FXbool fxezquantize(FXuchar* dst,const FXColor* src,FXColor* colormap,FXi
 // Xiaolin Wu's quantization method based on recursive partitioning
 extern FXbool fxwuquantize(FXuchar* dst,const FXColor* src,FXColor* colormap,FXint& actualcolors,FXint w,FXint h,FXint maxcolors);
 
-}
-
-using namespace FX;
-
 // X11 helpers
 #ifndef WIN32
 extern FXAPI Atom fxsendrequest(Display *display,Window window,Atom selection,Atom prop,Atom type,FXuint time);
 extern FXAPI Atom fxsendreply(Display *display,Window window,Atom selection,Atom prop,Atom target,FXuint time);
 extern FXAPI Atom fxsendtypes(Display *display,Window window,Atom prop,FXDragType* types,FXuint numtypes);
-extern FXAPI Atom fxrecvtypes(Display *display,Window window,Atom prop,FXDragType*& types,FXuint& numtypes);
+extern FXAPI Atom fxrecvtypes(Display *display,Window window,Atom prop,FXDragType*& types,FXuint& numtypes,FXbool del);
 extern FXAPI Atom fxsenddata(Display *display,Window window,Atom prop,Atom type,FXuchar* data,FXuint size);
 extern FXAPI Atom fxrecvdata(Display *display,Window window,Atom prop,Atom incr,Atom& type,FXuchar*& data,FXuint& size);
 #endif
@@ -96,10 +94,14 @@ extern FXAPI Atom fxrecvdata(Display *display,Window window,Atom prop,Atom incr,
 extern FXAPI HANDLE fxsendrequest(HWND window,HWND requestor,WPARAM type);
 extern FXAPI HANDLE fxsenddata(HWND window,FXuchar* data,FXuint size);
 extern FXAPI HANDLE fxrecvdata(HANDLE hMap,FXuchar*& data,FXuint& size);
-extern FXAPI unsigned int fxmodifierkeys();
+extern FXAPI FXuint fxmodifierkeys();
 extern UINT wkbGetCodePage();
 extern FXuint wkbMapKeyCode(UINT iMsg, WPARAM uVirtKey, LPARAM lParam);
 extern FXAPI FXbool wkbTranslateMessage(HWND hWnd, UINT iMsg, WPARAM wParam,LPARAM lParam);
+extern int (WINAPI *ToUnicodeEx)(UINT, UINT, const BYTE*, LPWSTR, int, UINT, HKL);
 #endif
 
+}
+
+
 
